/*
	File:		MoreControls.h

	Contains:	

	Written by:	Pete Gontier (PCG)

	Copyright:	Copyright (c) 1998 Apple Computer, Inc.

	Change History (most recent first):

         <5>      9/9/98    PCG     re-work import and export pragmas
         <4>      9/1/98    PCG     Universal Headers 3.2
         <3>     7/24/98    PCG	    coddle linker (C++, CFM-68K)
         <2>     7/24/98    PCG	    eliminate dependency on 'qd'
         <2>     6/23/98    PCG     add IsScrollBar
         <1>     6/16/98    PCG     initial checkin
*/


#pragma once

#include "MorePrefix.h"

#ifndef __CONTROLS__
#	include <Controls.h>
#endif

enum
{
	//////////////////////////////////////////////////////////////////
	//
	//	resource IDs of CDEFs (control definition proc resources)
	//
	//////////////////////////////////////////////////////////////////

	kControlOldButtonDefProcResID,
	kControlOldScrollBarDefProcResID,
	kControlBevelButtonDefProcResID,
	kControlSliderDefProcResID,
	kControlTriangleDefProcResID,
	kControlProgressBarDefProcResID,
	kControlLittleArrowsDefProcResID,
	kControlChasingArrowsDefProcResID,
	kControlTabsDefProcResID,
	kControlSeparatorLineDefProcResID,
	kControlGroupBoxDefProcResID,
	kControlImageWellDefProcResID,
	kControlPopupArrowDefProcResID,
	kControlUnluckyDefProcResID,
	kControlPlacardDefProcResID,
	kControlClockDefProcResID,
	kControlUserPaneDefProcResID,
	kControlEditTextDefProcResID,
	kControlStaticTextDefProcResID,
	kControlPictureDefProcResID,
	kControlIconDefProcResID,
	kControlWindowHeaderDefProcResID,
	kControlListBoxDefProcResID,
	kControlPushButtonDefProcResID,
	kControlScrollBarDefProcResID,
	kControlPopupButtonDefProcResID,
	kControlRadioGroupDefProcResID,
	kControlOldPopupMenuDefProcResID = 63
};

enum
{
	//////////////////////////////////////////////////////////////////
	//
	//	resource type for individual control font style record
	//
	//////////////////////////////////////////////////////////////////

	kMoreControls_ControlStyleType = FOUR_CHAR_CODE('cfsr')
};

enum
{
	//////////////////////////////////////////////////////////////////
	//
	//	gap between controls and from edges of the parent window
	//
	//////////////////////////////////////////////////////////////////

	MoreControls_kControlGap = (3 + 1) // 3 for focus, 1 for white space
};

enum
{
	//////////////////////////////////////////////////////////////////
	//
	//	alignments for AlignControl and AlignSubControls
	//
	//////////////////////////////////////////////////////////////////


	MoreControls_kAlignSystem,
	MoreControls_kAlignLeft,
	MoreControls_kAlignCenter,
	MoreControls_kAlignRight
};

typedef UInt8 MoreControls_tAlignment;

enum
{
	//////////////////////////////////////////////////////////////////
	//
	//	directions for MoveControlNear and SetControlQuadrant
	//
	//////////////////////////////////////////////////////////////////

	MoreControls_kDirectionNorthWest,
	MoreControls_kDirectionNorth,
	MoreControls_kDirectionNorthEast,
	MoreControls_kDirectionWest,
	MoreControls_kDirectionCenter,
	MoreControls_kDirectionEast,
	MoreControls_kDirectionSouthWest,
	MoreControls_kDirectionSouth,
	MoreControls_kDirectionSouthEast
};

typedef UInt8 MoreControls_tDirection;

//////////////////////////////////////////////////////////////////
//
//	tab control stuff for Appearance 1.0.1 or later
//
//////////////////////////////////////////////////////////////////

#if UNIVERSAL_INTERFACES_VERSION < 0x0310

enum
{
	kControlTabInfoTag			= 'tabi'						/* ControlTabInfoRec*/
};


enum {
	kControlTabInfoVersionZero	= 0
};

struct ControlTabInfoRec {
	SInt16 							version;					/* version of this structure.*/
	SInt16 							iconSuiteID;				/* icon suite to use. Zero indicates no icon*/
	Str255 							name;						/* name to be displayed on the tab*/
};
typedef struct ControlTabInfoRec ControlTabInfoRec;

#endif // UNIVERSAL_INTERFACES_VERSION < 0x0310

//////////////////////////////////////////////////////////////////
//
//	finally, the function prototypes
//
//////////////////////////////////////////////////////////////////

#ifdef __cplusplus
	extern "C" {
#endif

#pragma import on // for clients
#pragma export on // for building a library

pascal OSErr	OffsetControl							(ControlHandle, SInt16 h, SInt16 v);
pascal OSErr	InsetControl							(ControlHandle,SInt16);
pascal OSErr	ExpandControl							(ControlHandle);
pascal OSErr	EncloseSubControls						(ControlHandle);
pascal OSErr	AlignControl							(ControlHandle anchor, ControlHandle, MoreControls_tAlignment);
pascal OSErr	AlignSubControls						(ControlHandle, MoreControls_tAlignment);
pascal OSErr	SetBestControlRect						(ControlHandle, SInt16 *baseLineOffset);
pascal OSErr	MoveControlNear							(ControlHandle moveMe, ControlHandle near, MoreControls_tDirection);
pascal OSErr	SetControlQuadrant						(ControlHandle, MoreControls_tDirection);
pascal OSErr	MoreFindControlUnderMouse				(Point, WindowRef, ControlPartCode *, ControlHandle *);
pascal OSErr	GetStaticTextControlTextHeight			(ControlHandle, SInt16 *);
pascal OSErr	SetUserPaneDrawProc						(ControlHandle, ControlUserPaneDrawProcPtr);
pascal OSErr	SetUserPaneSetUpSpecialBackgroundProc	(ControlHandle, ControlUserPaneBackgroundProcPtr);
pascal OSErr	SetBevelButtonMenu						(ControlHandle,MenuRef);
pascal OSErr	SetBevelButtonMenuDelay					(ControlHandle,SInt32);
pascal OSErr 	SetProgressBarIndeterminate				(ControlHandle,Boolean);
pascal OSErr	SetPopUpButtonMenu						(ControlHandle, MenuRef);
pascal OSErr	Get1NewControl							(short resID, WindowRef, ControlHandle *);
pascal OSErr	Get1NewStaticTextControl				(short resID, WindowRef, ControlHandle *);
pascal OSErr	Get1NewPopupButtonControl				(short resID, WindowRef, ControlHandle *);
pascal OSErr	InvalControl							(ControlHandle);
pascal OSErr	CopyTabFontStyle						(ControlHandle oldTabs, ControlHandle newTabs);
pascal OSErr	ToggleControl							(ControlHandle);
pascal OSErr	IdleControlsInAllWindows				(void);
pascal OSErr	SetCursorAccordingToControl				(QDGlobalsPtr);
pascal OSErr	GetControlDefProcResID					(ControlHandle, short *);
pascal OSErr	MoreGetControlRegion					(ControlHandle, RgnHandle *);
pascal OSErr	SetControlTextFromResource				(ControlHandle, short textResID);
pascal OSErr	SetControlTextStyleFromResource			(ControlHandle, short styleResID);

pascal OSErr	InsertTab								(ControlHandle, ControlHandle *, UInt16 before);
pascal OSErr	RemoveTab								(ControlHandle, ControlHandle *, UInt16 which);
pascal OSErr	AppendTabs								(ControlHandle, UInt16);
pascal OSErr	TruncateTabs							(ControlHandle, UInt16);

pascal OSErr	SetTabIcon								(ControlHandle, ControlPartCode, short iconSuiteID);

pascal OSErr	IsScrollBar								(ControlHandle, Boolean *);

#pragma import reset // for clients
#pragma export reset // for building a library

#ifdef __cplusplus
	}
#endif
