/*
	File:		TransferProvider.c

	Contains:	Sample that shows how to use OTTransferProviderOwnership.

	Written by:	Quinn "The Eskimo!"

	Copyright:	 1997 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

	You may incorporate this sample code into your applications without
	restriction, though the sample code has been provided "AS IS" and the
	responsibility for its operation is 100% yours.  However, what you are
	not permitted to do is to redistribute the source as "DSC Sample Code"
	after having made changes. If you're going to re-distribute the source,
	we require that you make it clear in the source that the code was
	descended from Apple Sample Code, but that you've made changes.
*/

////////////////////////////////////////////////////////////
// Pick up general Open Transport stuff.

#include <OpenTransport.h>

////////////////////////////////////////////////////////////
// Pick up OT debugging macros.

#include <OTDebug.h>

////////////////////////////////////////////////////////////
// Get our own prototype.

#include "ProviderFactory.h"

////////////////////////////////////////////////////////////
// OTDebugStr is not defined in any of the interfaces,
// but it is in the libraries.

extern void OTDebugStr(char *str);

////////////////////////////////////////////////////////////

static Boolean gOTInited = false;

////////////////////////////////////////////////////////////
// Provider factory routines.

extern OSStatus FactoryCreateEndpoint(EndpointRef *ref, OTClient *currentOwner)
	// See comment in interface part.
	// In this example, we arbitrarily create an ADSP endpoint.  This
	// technique works for all types of endpoints.
{
	OSStatus err;
	
	OTAssert("FactoryCreateEndpoint: Parameter error", ref != nil);
	OTAssert("FactoryCreateEndpoint: Parameter error", currentOwner != nil);
	OTAssert("FactoryCreateEndpoint: Called but we weren't successfully inited.", gOTInited);

	*currentOwner = OTWhoAmI();
	*ref = OTOpenEndpoint(OTCreateConfiguration("adsp"), 0, nil, &err);

	return err;
}

////////////////////////////////////////////////////////////
// Initialisation and termination routines.

extern OSStatus InitProviderFactory(void)
	// See comment in interface part.
{
	OSStatus err;
	
	err = InitOpenTransport();
	gOTInited = (err == noErr);
	
	return err;
}

extern void CloseProviderFactory(void)
	// See comment in interface part.
{
	OTAssert("CloseProviderFactory: Called but we weren't successfully inited.", gOTInited);
	if (gOTInited) {
		CloseOpenTransport();
	}
}
